"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TonCatTransactionFetcher = void 0;
const TonCatClient_js_1 = require("../../../api_clients/TonCatClient.js");
const ton_core_1 = require("ton-core");
const Transaction_js_1 = require("../Transaction.js");
const Message_js_1 = require("../Message.js");
const WalletType_js_1 = require("../../wallets/WalletType.js");
const JettonWallet_js_1 = require("../../wallets/JettonWallet.js");
const UnknownWallet_js_1 = require("../../wallets/UnknownWallet.js");
const Wallet_js_1 = require("../../wallets/Wallet.js");
const ActionType_js_1 = require("../actions/ActionType.js");
const JettonInternalTransferAction_js_1 = require("../actions/JettonInternalTransferAction.js");
const JettonTransferAction_js_1 = require("../actions/JettonTransferAction.js");
const JettonTransferNotificationAction_js_1 = require("../actions/JettonTransferNotificationAction.js");
const JettonExcessesAction_js_1 = require("../actions/JettonExcessesAction.js");
class TonCatTransactionFetcher extends TonCatClient_js_1.TonCatClient {
    constructor(account) {
        super();
        this.account = account;
    }
    parseAction(actionFromResponse) {
        let type = actionFromResponse === null || actionFromResponse === void 0 ? void 0 : actionFromResponse.type;
        switch (type) {
            case ActionType_js_1.ActionType.jetton_internal_transfer:
                {
                    const amount = BigInt(actionFromResponse.amount);
                    const source = new UnknownWallet_js_1.UnknownWallet(ton_core_1.Address.parse(actionFromResponse.from));
                    const destination = new UnknownWallet_js_1.UnknownWallet(ton_core_1.Address.parse(actionFromResponse.response_address));
                    return new JettonInternalTransferAction_js_1.JettonInternalTransferAction(amount, source, destination);
                }
            case ActionType_js_1.ActionType.jetton_transfer:
                {
                    const amount = BigInt(actionFromResponse.amount);
                    let destination = null;
                    let response_destination = null;
                    if (actionFromResponse.destination)
                        destination = new UnknownWallet_js_1.UnknownWallet(ton_core_1.Address.parse(actionFromResponse.destination));
                    if (actionFromResponse.response_destination)
                        response_destination = new UnknownWallet_js_1.UnknownWallet(ton_core_1.Address.parse(actionFromResponse.response_destination));
                    return new JettonTransferAction_js_1.JettonTransferAction(amount, destination, response_destination);
                }
            case ActionType_js_1.ActionType.jetton_transfer_notification:
                {
                    const amount = BigInt(actionFromResponse.amount);
                    let sender = null;
                    if (actionFromResponse.sender)
                        sender = new UnknownWallet_js_1.UnknownWallet(ton_core_1.Address.parse(actionFromResponse.sender));
                    return new JettonTransferNotificationAction_js_1.JettonTransferNotificationAction(amount, sender);
                }
            case ActionType_js_1.ActionType.jetton_excesses:
                return new JettonExcessesAction_js_1.JettonExcessesAction();
            default:
                const msg = "Error: Action type: '" + actionFromResponse.type + "' is not implemented!";
                console.error(msg, actionFromResponse);
                throw new Error(msg);
        }
    }
    //было бы отлично это перенести в отдельный объект. Этот объект превращается в GodObject :\
    parseAccount(type, address) {
        let result = null;
        if (type == WalletType_js_1.WalletType.jetton_wallet)
            result = new JettonWallet_js_1.JettonWallet(ton_core_1.Address.parse(address));
        else if (type == WalletType_js_1.WalletType.wallet)
            result = new Wallet_js_1.Wallet(ton_core_1.Address.parse(address));
        else if (type == null)
            result = null;
        else {
            console.warn("Warning: Account type: '" + type + "' is not implemented! Address: " + address);
            result = new UnknownWallet_js_1.UnknownWallet(ton_core_1.Address.parse(address));
        }
        ;
        return result;
    }
    parseMessage(messageFromResponse) {
        let source = this.parseAccount(messageFromResponse.source_type, messageFromResponse.source);
        let destination = this.parseAccount(messageFromResponse.destination_type, messageFromResponse.destination);
        const value = BigInt(messageFromResponse.value);
        const actionData = messageFromResponse === null || messageFromResponse === void 0 ? void 0 : messageFromResponse.action;
        let action = null;
        if (actionData)
            action = this.parseAction(actionData);
        return new Message_js_1.Message(source, destination, value, messageFromResponse.op, messageFromResponse.bounced, action, messageFromResponse.comment);
    }
    //говнокод, мне не нравится, но работает
    //UPD: говнокод уменьшился
    fetchTransactions(limit = 20, offset = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const address = this.account.address;
            const response = yield this.get("contracts/address/" + address.toString() + "/transactions", { limit: limit, offset: offset });
            const data = response.data;
            let result = [];
            for (const item of data) {
                const time = new Date(item.utime * 1000);
                const fee = BigInt(item.fee);
                const out_msgs = new Array;
                for (const out_msg of item.out_msgs) {
                    out_msgs.push(this.parseMessage(out_msg));
                }
                const in_msg = this.parseMessage(item.in_msg);
                let transaction = new Transaction_js_1.Transaction(this.account, time, fee, item.hash, item.lt, item.compute_exit_code, item.action_result_code, in_msg, out_msgs);
                result.push(transaction);
            }
            return result;
        });
    }
}
exports.TonCatTransactionFetcher = TonCatTransactionFetcher;
